/**
 * Representação de uma entidade o ECS
 */
export declare abstract class Entity {
    static sequence(): number;
    /**
     * Lista de interessados sobre a atualiação dos componentes
     */
    private subscriptions;
    id: number;
    /**
     * Informa se a entidade está ativa
     */
    active: boolean;
    /**
     * Componentes por tipo
     *
     * ATENÇÃO! Apesar de a lista de componentes ser publica, não fazer modificação na mesma, utilizar os metodos add e
     * remove para tal
     */
    components: {
        [key: number]: Component<any>[];
    };
    constructor();
    /**
     * Permite aos interessados receber informação quando a lista de componentes dessa entidade receber atualização
     *
     * @param handler
     */
    subscribe(handler: () => void): () => void;
    /**
     * Adiciona um componente à entidade
     *
     * @param component
     */
    add(component: Component<any>): void;
    /**
     * Remove a referencia de um componente dessa entidade
     *
     * @param component
     */
    remove(component: Component<any>): void;
}
/**
 * Force typing
 */
export declare type ComponentClassType<P> = (new (data: P) => Component<P>) & {
    type: number;
    allFrom(entity: Entity): Component<P>[];
    oneFrom(entity: Entity): Component<P>;
};
/**
 * Informações sobre um componente
 */
export declare class Component<T> {
    /**
     * Registra uma nova classe de componente
     */
    static register<P>(): ComponentClassType<P>;
    /**
     * Obtem o id do tipo do componente
     */
    type: number;
    /**
     * Obtem o id do tipo do componente
     */
    data: T;
    constructor(type: number, data: T);
}
/**
 * Representação de um sistema no ECS
 */
export declare class System {
    /**
     * Implementação deve informar quais tipo de componentes esse sistema opera sobre
     */
    components: number[];
    /**
     * Invocado durante atualização
     * @param entity Identificador da entidade
     * @param components De uma entidade
     */
    update?(time: number, delta: number, entity: Entity): void;
    /**
     * Invocado sempre que uma entidade correspondente é adicionada no sistema, ou para todas as entidades existntes
     * quando o sistema é adicionado no mundo
     *
     * @param entity
     * @param components
     */
    enter?(entity: Entity): void;
    /**
     * Invocado sempre que uma entidade correspondente é removida do sistema, ou para todas as entidades existntes
     * quando o sistema é removido do mundo
     *
     * @param entity
     * @param components
     */
    exit?(entity: Entity): void;
}
/**
 * A propria definição do ECS. Também chamada de Admin ou Manager em outras implementações
 */
export default class ECS {
    static Entity: typeof Entity;
    static Component: typeof Component;
    static System: typeof System;
    private lastUpdate;
    /**
     * Todos os Systems existentes nesse mundo
     */
    private systems;
    /**
     * Todas as entidades existentes neste mundo
     */
    private entities;
    /**
     * Indexa os sistemas que devem ser executados para cada entidade
     */
    private entitiesSystems;
    /**
     * Guarda as subscrições realizadas para entidades
     */
    private entitiesSubscriptions;
    constructor(systems?: System[]);
    /**
     * Obtém uma entidade por id
     *
     * @param id
     */
    getEntity(id: number): Entity | undefined;
    /**
     * Adiciona uma entidade nesse mundo
     *
     * @param entity
     */
    addEntity(entity: Entity): void;
    /**
     * Remove uma entidade desse mundo
     *
     * @param idOrInstance
     */
    removeEntity(idOrInstance: number | Entity): void;
    /**
     * Adiciona o sistema nesse mundo
     *
     * @param system
     */
    addSystem(system: System): void;
    /**
     * Remove o sistema desse mundo
     *
     * @param system
     */
    removeSystem(system: System): void;
    /**
     * Realiza a atualização dos sistemas desse mundo
     */
    update(): void;
    /**
     * Faz a indexação de uma entidade
     *
     * @param entity
     */
    private indexEntity;
}
